package adelfeToolkit;

/*
 * AdelfeToolkit.java
 *
 * Created on 2 dcembre 2002, 10:10
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.Properties;

import adelfeToolkit.exceptions.InitSocketFailedException;
import adelfeToolkit.exceptions.ProcessFailedException;
import adelfeToolkit.gui.AdelfeStartScreen;
import adelfeToolkit.gui.AdelfeToolkitUI;
import adelfeToolkit.language.LanguageChangeSupport;
import adelfeToolkit.modules.DescriptionModule;
import adelfeToolkit.modules.ExampleModule;
import adelfeToolkit.modules.HelpModule;
import adelfeToolkit.modules.ManagerModule;
import adelfeToolkit.modules.SynthesisModule;
import adelfeToolkit.modules.Validator;
import adelfeToolkit.modules.WorkProductModule;
import adelfeToolkit.openTool.OpenToolConnector;
import adelfeToolkit.openTool.OpenToolMessageEvent;
import adelfeToolkit.openTool.OpenToolMessageHandler;
import adelfeToolkit.openTool.OpenToolMessageListener;

/** This is the main class of the Adelfe Toolkit. It is an executable class which
 * owns a main method.
 * @author IRIT - Toulouse (projet ADELFE)
 * @version 2.0
 */
public class AdelfeToolkit
implements java.io.Serializable,adelfeToolkit.utilities.LogWriter,
OpenToolMessageHandler{
  
  /** The help module of the Toolkit. */
  private HelpModule helpModule;
  
  /** The manager module of the Toolkit. */
  private ManagerModule managerModule;
  
  /** The description module of the Toolkit. */
  private DescriptionModule descriptionModule;
  
  /** The work product module of the Toolkit. */
  private WorkProductModule workProductModule;
  
  /** The example module of the Toolkit. */
  private ExampleModule exampleModule;
  
  /** The synthesis module of the Toolkit. */
  private SynthesisModule synthesisModule;
  
  /** The GUI that represents the Adelfe Toolkit. */
  private static AdelfeToolkitUI uI;
  
  /** The Validator that validates the process advancement and the production */  
  private Validator validator;
  
  /** The language in which the Toolkit has to be displayed. */
  private String language = LanguageChangeSupport.ENGLISH;
  
  /** the default directory of the project */  
  private static String DEFAULT_DIRECTORY = ".";
  
  /** the default name for the config directory */  
  private static String CONFIG_DIRECTORY_NAME = "config";
  
  /** The name of the configuration file of the toolkit */
  private static String CONFIG_FILE = "config.properties";
  
  /** the name of the log file */  
  private static String LOG_FILE = "adelfe.log";
  
  /** the default config file name */  
  private static String DEFAULT_CONFIG_FILE = "adelfeToolkit/resources/default";
  
  /** the default path to get html resources or images. */
  public static final String DEFAULT_PATH = "/adelfeToolkit/resources/";
  
  /** the path to OT */  
  private String openTool;
  
  /** the path to the Adequacy tool */  
  private String adequacy;
  
  /** the path to the drawing tool */  
  private String drawingTool;
  
  /** the project directory path */  
  private String projectDirectory;
  
  /** the path to the text editor */  
  private String textEditor;
  
  /** the path to the acrobat reader */  
  private String acrobatPath;
  
  /** the path to the previous opened file */
  private String previousFile;
  
  /** indicates if OT is running or not */  
  private boolean openToolRunning = false;
  
  /** A boolean that indicates if yes or not the activities are accessible by the
   * user.
   */
  private boolean allActivitiesAreAccessible = true;
  
  /** indicates if the Description Module is visible */  
  private boolean descriptionModuleIsVisible = true;
  
  /** indicates if the Example Module is visible */  
  private boolean exampleModuleIsVisible = true;
  
  /** indicates if the Manager Module is visible */  
  private boolean managerModuleIsVisible = true;
  
  /** indicates if the Document Module is visible */  
  private boolean documentModuleIsVisible = true;
  
  /** indicates if the Synthesis Module is visible */  
  private boolean synthesisModuleIsVisible = true;
  
  /** the socket port number */  
  private int socketPort = 8300;
  
  /** The OpenToolConnector tahtt manages the conection with OT */  
  private OpenToolConnector openToolConnector = new OpenToolConnector(this);
  
  /** the Properties corresponding to the user configuration */  
  private Properties personalConfig;
  
  /** the Properties corresponding to the default configuration */  
  private Properties defaultConfig;
  
  /** the File corresponding to the log file */  
  private File logFile;
  
  /** the FileWriter that writes in the log file */  
  private FileWriter logWriter;
  
  /** Creates a new instance of AdelfeToolkit */
  public AdelfeToolkit() {
    initLogFile();
    initConfig();
    initOpenToolConnector();
    managerModule = new ManagerModule(this);
    printOutLog("AdelfeToolkit::ManagerModule created");
    helpModule = new HelpModule(this);
    printOutLog("AdelfeToolkit::HelpModule created");
    descriptionModule = new DescriptionModule(this);
    printOutLog("AdelfeToolkit::DescriptionModule created");
    workProductModule = new WorkProductModule(this);
    printOutLog("AdelfeToolkit::WorkProductModule created");
    exampleModule = new ExampleModule(this);
    printOutLog("AdelfeToolkit::ExampleModule created");
    synthesisModule = new SynthesisModule(this);
    printOutLog("AdelfeToolkit::SynthesisModule created");
    managerModule.initializeActivities();
    uI = new AdelfeToolkitUI(this);
    validator = new Validator(this);
    initializeListeners();
    updateValidator();
    printOutLog("AdelfeToolkit::AdelfeTookit UI created");
    printOutLog("AdelfeToolkit::System initialized...OK\n");
  }
  
  /** this method initializes diffrent listeners such as OT log listeners or OT socket
   * listeners
   */  
  private void initializeListeners() {
    this.addOpenToolMessageListener(validator);
    this.addOpenToolMessageListener(uI);
    this.managerModule.addActionExitedListener(validator);
    this.managerModule.addActionExitedListener(uI);
    AdelfeToolkit.uI.getDescriptionModuleUI().addShortCutClickedListener(validator);
    AdelfeToolkit.uI.getDescriptionModuleUI().addShortCutClickedListener(uI);
  }
  
  /** this method calls the update method of the validator */  
  private void updateValidator(){
    validator.update();
  }
  
  /** This method sets the defaultConfig Properties to its initial values from
   * DEFAULT_CONFIG_FILE.
   */
  public void setConfigToDefaults() {
    defaultConfig = new Properties();
    java.util.ResourceBundle rb =
    java.util.ResourceBundle.getBundle(AdelfeToolkit.DEFAULT_CONFIG_FILE);
    
    for (Enumeration e = rb.getKeys();e.hasMoreElements();) {
      String s = (String)(e.nextElement());
      defaultConfig.setProperty(s,rb.getString(s));
    }
    
    printOutLog("setConfigToDefaults::Default property file loaded");
    personalConfig = new Properties(defaultConfig);
  }
  
  /** This method sets the properties of the Toolkit to the user's personal ones.
   */
  public void setPropertiesToPersonalProperties() {
    setOpenTool(personalConfig.getProperty("OpenTool"));
    setDrawingTool(personalConfig.getProperty("DrawingTool"));
    setProjectDirectory(personalConfig.getProperty("ProjectDirectory"));
    setTextEditor(personalConfig.getProperty("TextEditor"));
    setAllActivitiesAreAccessible(personalConfig.getProperty("AllActivitiesAreAccessible").equals("true"));
    setSocketPort((new Integer(personalConfig.getProperty("SocketPort"))).intValue());
    setAdequacy(personalConfig.getProperty("Adequacy"));
    setAcrobatPath(personalConfig.getProperty("AcrobatPath"));
    setDescriptionModuleIsVisible(personalConfig.getProperty("DescriptionModuleIsVisible").equals("true"));
    setManagerModuleIsVisible(personalConfig.getProperty("ManagerModuleIsVisible").equals("true"));
    setExampleModuleIsVisible(personalConfig.getProperty("ExampleModuleIsVisible").equals("true"));
    setDocumentModuleIsVisible(personalConfig.getProperty("DocumentModuleIsVisible").equals("true"));
    setSynthesisModuleIsVisible(personalConfig.getProperty("SynthesisModuleIsVisible").equals("true"));
    setLanguage(personalConfig.getProperty("Language"));
    setPreviousFile(personalConfig.getProperty("PreviousFile"));
    printOutLog("setPropertiesToPersonalProperties::Personal-configurationalized");
  }
  
  /** This methods loads the user's personal properties from the ./config directory if
   * possible, else its loads form the user's home directory.
   */
  public void initConfig() {
    setConfigToDefaults();
    try {
      // now load properties from last invocation
      String filesep = System.getProperty("file.separator");
      FileInputStream in = new FileInputStream(new java.io.File(AdelfeToolkit.DEFAULT_DIRECTORY+filesep+AdelfeToolkit.CONFIG_DIRECTORY_NAME+filesep+AdelfeToolkit.CONFIG_FILE));
      personalConfig.load(in);
      printOutLog("initConfig::Personal property file loaded from : "+AdelfeToolkit.DEFAULT_DIRECTORY+filesep+AdelfeToolkit.CONFIG_FILE);
      in.close();
    }
    catch (java.io.FileNotFoundException e) {
      try {
        // now load properties from last invocation
        String folder = System.getProperty("user.home");
        String filesep = System.getProperty("file.separator");
        FileInputStream in = new FileInputStream(new java.io.File(folder+filesep+AdelfeToolkit.CONFIG_FILE));
        personalConfig.load(in);
        printOutLog("initConfig::Personal property file loaded from : "+folder+filesep+AdelfeToolkit.CONFIG_FILE);
        in.close();
      }
      catch (java.io.FileNotFoundException e2) {
        printErrLog("initConfig::configuration file not found");
      }
      catch (java.io.IOException e2) {
        printErrLog("initConfig::enable to read configuration file");
      }
    }
    catch (java.io.IOException e) {
      printErrLog("initConfig::enable to read configuration file");
    }
    setPropertiesToPersonalProperties();
  }
  
  /** this method initalizes the log file */  
  private void initLogFile() {
    try {
      // now load properties from last invocation
      String filesep = System.getProperty("file.separator");
      logWriter = new FileWriter(new java.io.File(AdelfeToolkit.DEFAULT_DIRECTORY+filesep+AdelfeToolkit.LOG_FILE));
      printOutLog("initLogFile::Log file opened at : "+AdelfeToolkit.DEFAULT_DIRECTORY+filesep+AdelfeToolkit.LOG_FILE);
    }
    catch (java.io.FileNotFoundException e) {
      try {
        // now load properties from last invocation
        String folder = System.getProperty("user.home");
        String filesep = System.getProperty("file.separator");
        logWriter = new FileWriter(new java.io.File(folder+filesep+AdelfeToolkit.LOG_FILE));
        printOutLog("initLogFile::Log file opened at : "+folder+filesep+AdelfeToolkit.CONFIG_FILE);
      }
      catch (java.io.FileNotFoundException e2) {
        System.err.println("AdelfeToolkit::initLogFile::Log file folder not found");
      }
      catch (java.io.IOException e2) {
        System.err.println("AdelfeToolkit::initLogFile::enable to open log file");
      }
    }
    catch (java.io.IOException e) {
      System.err.println("AdelfeToolkit::initLogFile::enable to open log file");
    }
  }
  
  /** This method write the String s on the LOG_FILE and on the System.out.
   * @param s the String to write on the log file.
   */
  public void printlnLogFileOut(String s) {
    try{
      System.out.println(s);
      logWriter.write("OUT:"+new java.util.Date(System.currentTimeMillis())+"\t"+s+"\n");
    }
    catch(IOException e) {
      System.err.println("AdelfeToolkit::printlnLogFileOut::Enable to write on log file.");
    }
  }
  
  /** This method write the String s on the LOG_FILE and on the System.err.
   * @param s the String to write on the LOG_FILE as an error.
   */
  public void printlnLogFileErr(String s) {
    try{
      System.err.println(s);
      logWriter.write("ERR:"+new java.util.Date(System.currentTimeMillis())+"\t"+s+"\n");
    }
    catch(IOException e) {
      System.err.println("AdelfeToolkit::printlnLogFileErr::Enable to write on log file.");
    }
  }
  
  /** This method closes the log file. */
  public void closeLogFile() {
    try{
      System.out.println("AdelfeToolkit::closeLogFile::Log file successfully closed");
      logWriter.write("\n============= END OF LOG FILE ============\n");
      logWriter.close();
    }
    catch(IOException e) {
      System.err.println("AdelfeToolkit::closeLogFile::Enable to close log file.");
    }
  }
  
  /** This method initialize the OpenToolConncetor that enable communication with
   * OpenTool.
   */
  public void initOpenToolConnector(){
  }
  
  /** This method launch the OpenTool program.
   * @throws ProcessFailedException This exception indicates that the process that support OpenTool failed.
   * @throws InitSocketFailedException This exception indicates that the socket intialization to communicate with
   * OpenTool failed.
   * @throws UnknownHostException This exception indicates that the host to open the socket is erroneous.
   * @return the returned boolean value indicates if OpenTool has been opened (if opened then
   * true else false).
   */
  public boolean runOpenTool() throws ProcessFailedException, InitSocketFailedException, UnknownHostException {
    boolean ret = isOpenToolRunning();
    if (!ret){
      try {
        String comm = getOpenTool()+" -l eng Main=uml";
        String filesep = System.getProperty("file.separator");
        String dir = managerModule.getCurrentProject().getDirectory().getFile()
        +managerModule.getCurrentProject().getActionName()
        +"_OT";
        File dirOT = new File(dir);
        String f = dirOT.getAbsolutePath()
        +filesep
        +managerModule.getCurrentProject().getActionName()
        +".otu";
        java.io.File file = new java.io.File(f);
        if (file.exists()) {
          comm += " "+file.getAbsolutePath();
        }
        System.out.println(comm);
        java.lang.Process p  = Runtime.getRuntime().exec(comm);
        //p.destroy();
        setOpenToolRunning(true);
        printOutLog("runOpenTool::OpenTool is now running.");
      }
      catch (IOException e) {
        printErrLog("runOpenTool::OpenTool failed to run.");
        throw new ProcessFailedException("Process Failed",e.getCause());
      }
      try {
        getOpenToolConnector().initSocket();
        printOutLog("runOpenTool::Socket is now initialized.");
      }
      catch (IOException e2) {
        throw new InitSocketFailedException("Socket Initialization Failed",e2.getCause());
      }
      return isOpenToolRunning();
    }
    else {
      if (testOpenToolConnection()) {
        return true;
      }
      else {
        setOpenToolRunning(false);
        return false;
      }
    }
  }
  
  /** this method is only a test method for the connectino with OT
   * @return indicates if the test is successful or not
   */  
  private boolean testOpenToolConnection() {
    boolean ret = false;
    try {
      getOpenToolConnector().send("TRANSCRIPT(\"Test from AdelfeToolkit\")\n\014");
      ret = true;
    }
    catch(IOException e) {
    }
    return ret;
  }
  
  /** This method launch the AMAS adequacy tool.
   * @throws ProcessFailedException This exception indicates that the process that support the AMAS adequacy tool failed.
   */
  public void runAMASAdeq() throws ProcessFailedException {
//    try {
//      String filesep = System.getProperty("file.separator");
//      System.out.println(getJrePath()+filesep+getAdequacy());
//      java.lang.Process p  = Runtime.getRuntime().exec(getJrePath()+filesep+"javaw -jar "+getAdequacy());
//      //p.destroy();
//      printOutLog("runOpenTool::Adelfe Adequacy Tool is now running.");
//    }
//    catch (IOException e) {
//      new ProcessFailedException("Process Failed",e.getCause());
//    }
  }
  
  /** This method stores the path to OpenTool in the config file. */
  public void saveOpenTool() {
    personalConfig.put("OpenTool",getOpenTool());
  }
  
  /** This method stores the path to AMAS adequacy in the config file. */
  public void saveAdequacy() {
    personalConfig.put("Adequacy",getAdequacy());
  }
  
  /** This method save the value of the allActivitiesAreAccessible option in the
   * config file.
   */
  public void saveAllActivitiesAreAccessible() {
    String res;
    if (isAllActivitiesAreAccessible()) {res="true";} else {res="false";};
    personalConfig.put("AllActivitiesAreAccessible",res);
  }
  
  /** This method stores the working path, called project directory, in the config file. */
  public void saveProjectDirectory() {
    personalConfig.put("ProjectDirectory",getProjectDirectory());
  }
  
  /** This method stores the path to the Text editor in the config file. */
  public void saveTextEditor() {
    personalConfig.put("TextEditor",getTextEditor());
  }
  
  /** This method stores the path to the Drawing tool in the config file. */
  public void saveDrawingTool() {
    personalConfig.put("DrawingTool",getDrawingTool());
  }
  
  /** This method stores the path to the Acrobat Reader in the config file. */
  public void saveAcrobatPath() {
    personalConfig.put("AcrobatPath", getAcrobatPath());
  }
  
  /** This method stores the path to the previous opened file in the config file. */
  public void savePreviousFile() {
    personalConfig.put("PreviousFile", getPreviousFile());
  }
  
  /** This method save the value of the descriptionModuleIsVisible option in the
   * config file.
   */
  public void saveDescriptionModuleIsVisible() {
    String res;
    if (isDescriptionModuleIsVisible()) {res="true";} else {res="false";};
    personalConfig.put("DescriptionModuleIsVisible",res);
  }
  
  /** This method save the value of the documentModuleIsVisible option in the
   * config file.
   */
  public void saveDocumentModuleIsVisible() {
    String res;
    if (isDocumentModuleIsVisible()) {res="true";} else {res="false";};
    personalConfig.put("DocumentModuleIsVisible",res);
  }
  
  /** This method save the value of the exampleModuleIsVisible option in the
   * config file.
   */
  public void saveExampleModuleIsVisible() {
    String res;
    if (isExampleModuleIsVisible()) {res="true";} else {res="false";};
    personalConfig.put("ExampleModuleIsVisible",res);
  }
  
  /** This method save the value of the synthesisModuleIsVisible option in the
   * config file.
   */
  public void saveSynthesisModuleIsVisible() {
    String res;
    if (isSynthesisModuleIsVisible()) {res="true";} else {res="false";};
    personalConfig.put("SynthesisModuleIsVisible",res);
  }
  
  /** This method save the value of the managerModuleIsVisible option in the
   * config file.
   */
  public void saveManagerModuleIsVisible() {
    String res;
    if (isManagerModuleIsVisible()) {res="true";} else {res="false";};
    personalConfig.put("ManagerModuleIsVisible",res);
  }
  
  /** This method stores the properties (pathes, options, ...) in a config file. This
   * config file will be either in the current path (if accessible) or in the user
   * home directory.
   */
  public void saveProperties() {
    try {
      String folder = System.getProperty("user.home");
      String filesep = System.getProperty("file.separator");
      File f = new File(AdelfeToolkit.DEFAULT_DIRECTORY+filesep+AdelfeToolkit.CONFIG_DIRECTORY_NAME);
      f.mkdirs();
      if (f.exists()){
        FileOutputStream out = new FileOutputStream(f.getPath()+filesep+AdelfeToolkit.CONFIG_FILE);
        personalConfig.store(out, "AdelfeToolkit Property File");
        out.close();
        printOutLog("saveProperties::Properties successfully saved to :"+f.getPath()+filesep+AdelfeToolkit.CONFIG_FILE);
      }
      else {
        FileOutputStream out = new FileOutputStream(folder+filesep+AdelfeToolkit.CONFIG_FILE);
        personalConfig.store(out, "AdelfeToolkit Property File");
        out.close();
        printOutLog("saveProperties::Properties successfully saved to :"+folder+filesep+AdelfeToolkit.CONFIG_FILE);
      }
    }
    catch (java.io.FileNotFoundException e){
      printErrLog("saveProperties::Personal property file not found");
    }
    catch (java.io.IOException e) {
      printErrLog("saveProperties::Not able to write property file");
    }
  }
  
  /** this method is called when the user want to stop the AdelfeToolkit. It does
   * these three main operations : close th e connections with OpenTool, save the
   * properties and close the log file.
   */
  public void exit() {
    try {
      this.openToolConnector.endConnection();
    } catch (IOException e) {
      this.printErrLog("exit::"+e);
    }
    saveProperties();
    closeLogFile();
  }
  
  /** This is the main method which is runned for the execution of the Toolkit.
   * @param args String value that contains the main parameters.
   */
  public static void main(String[] args) {
    AdelfeStartScreen s = new AdelfeStartScreen();
    java.lang.Thread t = new java.lang.Thread(s);
    t.start();
    
    AdelfeToolkit at = new AdelfeToolkit();
    
    s.setOwner(at);
    uI.toBack();
    s.hide();
    uI.show();
    uI.toFront();
    uI.setFocusable(true);
    uI.setFocusableWindowState(true);
    uI.setFocusCycleRoot(true);
    uI.setEnabled(true);
    uI.selectFocus();
    uI.refresh();
  }
  
  /** Getter for property descriptionModule.
   * @return Value of property descriptionModule.
   *
   */
  public DescriptionModule getDescriptionModule() {
    return descriptionModule;
  }
  
  /** Setter for property descriptionModule.
   * @param descriptionModule New value of property descriptionModule.
   *
   */
  public void setDescriptionModule(DescriptionModule descriptionModule) {
    this.descriptionModule = descriptionModule;
  }
  
  /** Getter for property exampleModule.
   * @return Value of property exampleModule.
   *
   */
  public ExampleModule getExampleModule() {
    return exampleModule;
  }
  
  /** Setter for property exampleModule.
   * @param exampleModule New value of property exampleModule.
   *
   */
  public void setExampleModule(ExampleModule exampleModule) {
    this.exampleModule = exampleModule;
  }
  
  /** Getter for property helpModule.
   * @return Value of property helpModule.
   *
   */
  public HelpModule getHelpModule() {
    return helpModule;
  }
  
  /** Setter for property helpModule.
   * @param helpModule New value of property helpModule.
   *
   */
  public void setHelpModule(HelpModule helpModule) {
    this.helpModule = helpModule;
  }
  
  /** Getter for property language.
   * @return Value of property language.
   *
   */
  public java.lang.String getLanguage() {
    return language;
  }
  
  /** Setter for property language.
   * @param language New value of property language.
   *
   */
  public void setLanguage(java.lang.String language) {
    this.language = language;
  }
  
  /** Getter for property managerModule.
   * @return Value of property managerModule.
   *
   */
  public ManagerModule getManagerModule() {
    return managerModule;
  }
  
  /** Setter for property managerModule.
   * @param managerModule New value of property managerModule.
   *
   */
  public void setManagerModule(ManagerModule managerModule) {
    this.managerModule = managerModule;
  }
  
  /** Getter for property synthesisModule.
   * @return Value of property synthesisModule.
   *
   */
  public SynthesisModule getSynthesisModule() {
    return synthesisModule;
  }
  
  /** Setter for property synthesisModule.
   * @param synthesisModule New value of property synthesisModule.
   *
   */
  public void setSynthesisModule(SynthesisModule synthesisModule) {
    this.synthesisModule = synthesisModule;
  }
  
  /** Getter for property uI.
   * @return Value of property uI.
   *
   */
  public AdelfeToolkitUI getUI() {
    return uI;
  }
  
  /** Setter for property uI.
   * @param uI New value of property uI.
   *
   */
  public void setUI(AdelfeToolkitUI uI) {
    AdelfeToolkit.uI = uI;
  }
  
  /** Getter for property workProductModule.
   * @return Value of property workProductModule.
   *
   */
  public WorkProductModule getWorkProductModule() {
    return workProductModule;
  }
  
  /** Setter for property workProductModule.
   * @param workProductModule New value of property workProductModule.
   *
   */
  public void setWorkProductModule(WorkProductModule workProductModule) {
    this.workProductModule = workProductModule;
  }
  
  /** Getter for property openTool.
   * @return Value of property openTool.
   *
   */
  public java.lang.String getOpenTool() {
    return openTool;
  }
  
  /** Setter for property openTool.
   * @param openTool New value of property openTool.
   *
   */
  public void setOpenTool(java.lang.String openTool) {
    this.openTool = openTool;
  }
  
  /** Getter for property textEditor.
   * @return Value of property textEditor.
   *
   */
  public java.lang.String getTextEditor() {
    return textEditor;
  }
  
  /** Setter for property textEditor.
   * @param textEditor New value of property textEditor.
   *
   */
  public void setTextEditor(java.lang.String textEditor) {
    this.textEditor = textEditor;
  }
  
  /** Getter for property projetcDirectory.
   * @return Value of property projetcDirectory.
   *
   */
  public java.lang.String getProjectDirectory() {
    return projectDirectory;
  }
  
  /** Setter for property projetcDirectory.
   * @param projetcDirectory New value of property projetcDirectory.
   *
   */
  public void setProjectDirectory(java.lang.String projetcDirectory) {
    this.projectDirectory = projetcDirectory;
  }
  
  /** Getter for property allActivitiesAreAccessible.
   * @return Value of property allActivitiesAreAccessible.
   *
   */
  public boolean isAllActivitiesAreAccessible() {
    return allActivitiesAreAccessible;
  }
  
  /** Setter for property allActivitiesAreAccessible.
   * @param allActivitiesAreAccessible New value of property allActivitiesAreAccessible.
   *
   */
  public void setAllActivitiesAreAccessible(boolean allActivitiesAreAccessible) {
    this.allActivitiesAreAccessible = allActivitiesAreAccessible;
  }
  
  /** Getter for property adequacy.
   * @return Value of property adequacy.
   *
   */
  public java.lang.String getAdequacy() {
    return adequacy;
  }
  
  /** Setter for property adequacy.
   * @param adequacy New value of property adequacy.
   *
   */
  public void setAdequacy(java.lang.String adequacy) {
    this.adequacy = adequacy;
  }
  
  /** Getter for property drawingTool.
   * @return Value of property drawingTool.
   *
   */
  public java.lang.String getDrawingTool() {
    return drawingTool;
  }
  
  /** Setter for property drawingTool.
   * @param drawingTool New value of property drawingTool.
   *
   */
  public void setDrawingTool(java.lang.String drawingTool) {
    this.drawingTool = drawingTool;
  }
  
  /** Getter for property OpenToolRunning.
   * @return Value of property OpenToolRunning.
   *
   */
  public boolean isOpenToolRunning() {
    return openToolRunning;
  }
  
  /** Setter for property OpenToolRunning.
   * @param OpenToolRunning New value of property OpenToolRunning.
   *
   */
  public void setOpenToolRunning(boolean OpenToolRunning) {
    this.openToolRunning = OpenToolRunning;
  }
  
  /** Getter for property openToolConnector.
   * @return Value of property openToolConnector.
   *
   */
  public adelfeToolkit.openTool.OpenToolConnector getOpenToolConnector() {
    return openToolConnector;
  }
  
  /** Setter for property openToolConnector.
   * @param openToolConnector New value of property openToolConnector.
   *
   */
  public void setOpenToolConnector(adelfeToolkit.openTool.OpenToolConnector openToolConnector) {
    this.openToolConnector = openToolConnector;
  }
  
  /** Getter for property socketPort.
   * @return Value of property socketPort.
   *
   */
  public int getSocketPort() {
    return socketPort;
  }
  
  /** Setter for property socketPort.
   * @param socketPort New value of property socketPort.
   *
   */
  public void setSocketPort(int socketPort) {
    this.socketPort = socketPort;
  }
  
  /** This method prints a string on the System.err.
   * @param s The string to print.
   */
  public void printErrLog(String s) {
    printlnLogFileErr("AdelfeToolkit::"+s);
  }
  
  /** This method prints a string on the System.out.
   * @param s The string to print.
   */
  public void printOutLog(String s) {
    printlnLogFileOut("AdelfeToolkit::"+s);
  }
  
  /** Getter for property acrobatPath.
   * @return Value of property acrobatPath.
   *
   */
  public java.lang.String getAcrobatPath() {
    return acrobatPath;
  }
  
  /** Setter for property acrobatPath.
   * @param acrobatPath the new path
   */
  public void setAcrobatPath(java.lang.String acrobatPath) {
    this.acrobatPath = acrobatPath;
  }
  
  /** Getter for property descriptionModuleIsVisible.
   * @return Value of property descriptionModuleIsVisible.
   *
   */
  public boolean isDescriptionModuleIsVisible() {
    return descriptionModuleIsVisible;
  }
  
  /** Setter for property descriptionModuleIsVisible.
   * @param descriptionModuleIsVisible New value of property descriptionModuleIsVisible.
   *
   */
  public void setDescriptionModuleIsVisible(boolean descriptionModuleIsVisible) {
    this.descriptionModuleIsVisible = descriptionModuleIsVisible;
  }
  
  /** Getter for property exampleModuleIsVisible.
   * @return Value of property exampleModuleIsVisible.
   *
   */
  public boolean isExampleModuleIsVisible() {
    return exampleModuleIsVisible;
  }
  
  /** Setter for property exampleModuleIsVisible.
   * @param exampleModuleIsVisible New value of property exampleModuleIsVisible.
   *
   */
  public void setExampleModuleIsVisible(boolean exampleModuleIsVisible) {
    this.exampleModuleIsVisible = exampleModuleIsVisible;
  }
  
  /** Getter for property managerModuleIsVisible.
   * @return Value of property managerModuleIsVisible.
   *
   */
  public boolean isManagerModuleIsVisible() {
    return managerModuleIsVisible;
  }
  
  /** Setter for property managerModuleIsVisible.
   * @param managerModuleIsVisible New value of property managerModuleIsVisible.
   *
   */
  public void setManagerModuleIsVisible(boolean managerModuleIsVisible) {
    this.managerModuleIsVisible = managerModuleIsVisible;
  }
  
  /** Getter for property documentModuleIsVisible.
   * @return Value of property documentModuleIsVisible.
   *
   */
  public boolean isDocumentModuleIsVisible() {
    return documentModuleIsVisible;
  }
  
  /** Setter for property documentModuleIsVisible.
   * @param documentModuleIsVisible New value of property documentModuleIsVisible.
   *
   */
  public void setDocumentModuleIsVisible(boolean documentModuleIsVisible) {
    this.documentModuleIsVisible = documentModuleIsVisible;
  }
  
  /** Getter for property synthesisModuleIsVisible.
   * @return Value of property synthesisModuleIsVisible.
   *
   */
  public boolean isSynthesisModuleIsVisible() {
    return synthesisModuleIsVisible;
  }
  
  /** Setter for property synthesisModuleIsVisible.
   * @param synthesisModuleIsVisible New value of property synthesisModuleIsVisible.
   *
   */
  public void setSynthesisModuleIsVisible(boolean synthesisModuleIsVisible) {
    this.synthesisModuleIsVisible = synthesisModuleIsVisible;
  }
  
  /** This method adds a new listener of Open Tool Message to the listeners list by
   * adding it to the listeners list of the OpenToolConnector
   * @param listener the listener to add
   */  
  public void addOpenToolMessageListener(OpenToolMessageListener listener) {
    this.openToolConnector.addOpenToolMessageListener(listener);
  }
  
  /** this method send an OpenToolMessageReceived event to the listeners of the list
   * @param evt the event to send
   */  
  public void fireOpenToolMessageReceived(OpenToolMessageEvent evt) {
    this.openToolConnector.fireOpenToolMessageReceived(evt);
  }
  
  /** This method removes a new listener of Open Tool Message from the listeners list by
   * removing it to the listeners list of the OpenToolConnector
   * @param listener the listener to remove
   */  
  public void removeOpenToolMessageListener(OpenToolMessageListener listener) {
    this.openToolConnector.removeOpenToolMessageListener(listener);
  }
  
  /** This method sets the state of the WorkProduct named wpName to the given state
   * @param wpName the name of the work product
   * @param state the new state
   */  
  public void setWPState(String wpName, String state) {
    workProductModule.setWPState(wpName, state);
  }
  
  /** This method sets the accessibility of the WorkProduct named wpName to the given
   * value
   * @param wpName the name of the work product
   * @param acces the new accessibility
   */  
  public void setWPAcces(String wpName, boolean acces) {
    workProductModule.setWPAcces(wpName, acces);
    
  }
  
  /** This method sets the producted field of the WorkProduct named wpName to the given
   * value
   * @param wpName the name of the work product
   * @param prod the new value for producted field
   */  
  public void setWPProduc(String wpName, boolean prod) {
    workProductModule.setWPProduc(wpName, prod);
    
  }
  
  /** This method sets the completed field of the ActionToDo named actName to the given
   * value
   * @param actName the name of the ActinoToDo
   * @param comp the new value for completed field
   */  
  public void setActCompl(String actName, boolean comp) {
    managerModule.setActCompl(actName, comp);
    
  }
  
  /** This method sets the accessibility of the ActionToDo named actName to the given
   * value
   * @param actName the name of the ActionToDo
   * @param acces the new value for accessible field
   */  
  public void setActAcces(String actName, boolean acces) {
    managerModule.setActAcces(actName, acces);
    
  }
  
  /** Getter for property previousFile.
   * @return Value of property previousFile.
   *
   */
  public java.lang.String getPreviousFile() {
    return previousFile;
  }
  
  /** Setter for property previousFile.
   * @param previousFile New value of property previousFile.
   *
   */
  public void setPreviousFile(java.lang.String previousFile) {
    this.previousFile = previousFile;
  }
  
}
