package adelfeToolkit.openTool;

/**
 *
 * @author  IRIT - Toulouse (projet ADELFE)
 */
import java.io.*;
import java.net.*;
import adelfeToolkit.AdelfeToolkit;
import java.util.Timer;


/** This class manages connections with OpenTool. */
public class OpenToolConnector implements OpenToolMessageHandler{
  
  /** the socket to communicate with OT */  
  private Socket socket;
  
  /** the stream to write into */  
  private OutputStreamWriter wst;
  
  /** the name of the host of the socket */  
  private String host = "localhost";
  
  /** the port of the socket */  
  private int port = 8300;
  
  /** the socket reader */  
  private OpenToolSocketReader otr;
  
  /** the log reader */  
  private OpenToolLogReader olr;
  
  /** the synthesis requester */  
  private OpenToolSynthesisRequester otsr;
  
  /** the stream to read into */  
  private InputStreamReader rst;
  
  /** the <CODE>AdelfeToolkit</CODE> that owns the <CODE>OpenToolConnector</CODE> */  
  private AdelfeToolkit owner;
  
  /** the timer for the log reader */  
  private Timer timer;
  
  /** the timer for the synthesis requester */  
  private Timer timer2;
  
  /** Creates a new instance of <CODE>OpenToolConnector</CODE>.
   * @param t the owner
   */
  public OpenToolConnector(AdelfeToolkit t) {
    owner = t;
    setOlr(new OpenToolLogReader(this));
    setOtr(new OpenToolSocketReader(rst));
    setOtsr(new OpenToolSynthesisRequester(this));
    
  }
  
  /** This method initialize the socket by which OpenTool and the <CODE>AdelfeToolkit </CODE>will
   * communicate.
   * @throws UnknownHostException this exception is raised if the socket ID is erroneous
   * @throws IOException this exception is raised if an error occur when writing or reading the socket
   */
  public void initSocket() throws UnknownHostException,IOException {
    System.out.println("OpenToolConnector::initSocket::Connecting...\n");
    setSocket(new Socket(host, port));
    setWst(new OutputStreamWriter(socket.getOutputStream()));
    rst = new InputStreamReader(socket.getInputStream());
    this.getOtr().setBr(rst);
    try {
      otr.start();
      timer = new Timer();
      timer.schedule(olr,0,1*200);
      timer2 = new Timer();
      timer2.schedule(otsr,0,1*1000);
    } catch (java.lang.IllegalThreadStateException e) {
    }    
  }
  
  /** This method close the input and output streams and the socket
   * @throws IOException Thsi execption is raised if a stream is already closed
   */
  public void closeAll() throws IOException{
    if (wst != null) {
      this.wst.close();
    }
    if (rst != null) {
      this.rst.close();
    }
    if (socket != null) {
      this.socket.close();
    }
  }
  
  /** this method makes the <CODE>OpenToolReader</CODE> sleep. */
  public void stopReader() {
    try {
      OpenToolSocketReader.sleep(100);
    } catch (java.lang.InterruptedException e) {
    }
  }
  
  /** This method is used to stop a connection : it closes the streams and socket and
   * the <CODE>OpenToolReader</CODE> is stopped.
   * @throws IOException this exception is raised by the closure of streams.
   */
  public void endConnection() throws IOException{
    if (this.timer != null) {
      this.timer.cancel();
    }
    if (this.timer != null) {
      this.timer2.cancel();
    }
    this.stopReader();
    this.closeAll();
  }
  
  /** This method is used to debug the class. */
  public void test() {
    try{
      wst.write("TRANSCRIPT(\"Hello from Java\")\n\014");
      wst.flush();
      wst.write("THIS.system.getRoot();$NO(ALL UMLUseCaseDiagram[name=\"nom_projet_UCD\"]) IF TRUE:{NEW UMLUseCaseDiagram.{initialize();name:=\"nom_projet_UCD\";diagram_title:=name;display_title:=TRUE;system:=THIS.system;};THIS.system.getRoot();ALL UMLUseCaseDiagram[name=\"nom_projet_UCD\"].open} IF FALSE:{INFO(\"Diagram already exists\");ALL UMLUseCaseDiagram[name=\"nom_projet_UCD\"].open}\n\014");
      wst.flush();
      //System.out.println((char)isr.read());
    }
    catch (IOException e) {System.out.println(e+"\n");}
  }
  
  /** This main method is used to test the class.
   * @param args arguments for the method (useless)
   */
  //  static public void main(String [] args) {
  //    OpenToolConnector otc = new OpenToolConnector(owner);
  //    try {
  //      otc.initSocket();
  //    }
  //    catch (UnknownHostException e) {System.err.println("OpenToolConnector::initSocket::UnknownHostException");}
  //    catch (IOException e) {System.err.println("OpenToolConnector::initSocket::IOException : "+e);}
  //    otc.test();
  //  }
  
  /** This method is used to send a message to OpenTool via the write stream.
   * @param s the String that represents the message to send
   * @throws IOException this exception is raised if an error occur during the writing
   */
  public void send(String s) throws IOException{
    if ((wst == null)) {
      try{
        System.out.println("OpenToolConnector::send::socket misinitialized. Trying to reinitialize...");
        initSocket();
      }
      catch (UnknownHostException e) {
        System.err.println("OpenToolConnector::send::UnknownHostException during socket initialization - "+e);
      }
    }
    if (!socket.isOutputShutdown()) {
      wst.write(s);
      wst.flush();
    }
  }
  
  /** This method is used if the send method failed to resend a message.
   * @param s the message to resend
   */
  public void reSend(String s) {
    try{
      if ((wst == null)) {
        try{
          System.out.println("OpenToolConnector::reSend::socket misinitialized. Trying to reinitialize...");
          initSocket();
        }
        catch (UnknownHostException e) {
          System.err.println("OpenToolConnector::reSend::UnknownHostException during socket initialization");
        }
      }
      wst.write(s);
      wst.flush();
    }
    catch (IOException e) {
      System.err.println("OpenToolConnector::reSend::IOException : OpenTool has not responded yet. Aborting communication...");
    }
  }
  
  /** Getter for property socket.
   * @return Value of property socket.
   *
   */
  public java.net.Socket getSocket() {
    return socket;
  }
  
  /** Setter for property socket.
   * @param socket New value of property socket.
   *
   */
  public void setSocket(java.net.Socket socket) {
    this.socket = socket;
  }
  
  /** Getter for property wst.
   * @return Value of property wst.
   *
   */
  public java.io.OutputStreamWriter getWst() {
    return wst;
  }
  
  /** Setter for property wst.
   * @param wst New value of property wst.
   *
   */
  public void setWst(java.io.OutputStreamWriter wst) {
    this.wst = wst;
  }
  
  /** Getter for property host.
   * @return Value of property host.
   *
   */
  public java.lang.String getHost() {
    return host;
  }
  
  /** Setter for property host.
   * @param host New value of property host.
   *
   */
  public void setHost(java.lang.String host) {
    this.host = host;
  }
  
  /** Getter for property port.
   * @return Value of property port.
   *
   */
  public int getPort() {
    return port;
  }
  
  /** Setter for property port.
   * @param port New value of property port.
   *
   */
  public void setPort(int port) {
    this.port = port;
  }
  
  /** Getter for property otr.
   * @return Value of property otr.
   *
   */
  public adelfeToolkit.openTool.OpenToolSocketReader getOtr() {
    return otr;
  }
  
  /** Setter for property otr.
   * @param otr New value of property otr.
   *
   */
  public void setOtr(adelfeToolkit.openTool.OpenToolSocketReader otr) {
    this.otr = otr;
  }
  
  /** This method calls the <CODE>addOpenToolMessageListener</CODE> method of the OpenToolReader.
   * @param listener the listener to add to the listener list.
   * @see adelfeToolkit.openTool.OpenToolMessageHandler
   */
  public void addOpenToolMessageListener(OpenToolMessageListener listener) {
    otr.addOpenToolMessageListener(listener);
    olr.addOpenToolMessageListener(listener);
  }
  
  /** This method calls the <CODE>fireOpenToolMessageReceived</CODE> method of the OpenToolReader.
   * @param evt The OpenToolMessageEvent sent.
   * @see adelfeToolkit.openTool.OpenToolMessageHandler
   */
  public void fireOpenToolMessageReceived(OpenToolMessageEvent evt) {
    otr.fireOpenToolMessageReceived(evt);
    olr.fireOpenToolMessageReceived(evt);
  }
  
  /** This method calls the <CODE>removeOpenToolMessageListener</CODE> method of the OpenToolReader.
   * @param listener the listener to remove.
   * @see adelfeToolkit.openTool.OpenToolMessageHandler
   */
  public void removeOpenToolMessageListener(OpenToolMessageListener listener) {
    otr.removeOpenToolMessageListener(listener);
    olr.removeOpenToolMessageListener(listener);
  }
  
  /** Getter for property owner.
   * @return Value of property owner.
   *
   */
  public adelfeToolkit.AdelfeToolkit getOwner() {
    return owner;
  }
  
  /** Setter for property owner.
   * @param owner New value of property owner.
   *
   */
  public void setOwner(adelfeToolkit.AdelfeToolkit owner) {
    this.owner = owner;
  }
  
  /** Getter for property olr.
   * @return Value of property olr.
   *
   */
  public adelfeToolkit.openTool.OpenToolLogReader getOlr() {
    return olr;
  }
  
  /** Setter for property olr.
   * @param olr New value of property olr.
   *
   */
  public void setOlr(adelfeToolkit.openTool.OpenToolLogReader olr) {
    this.olr = olr;
  }
  
  /** Getter for property otsr.
   * @return Value of property otsr.
   *
   */
  public adelfeToolkit.openTool.OpenToolSynthesisRequester getOtsr() {
    return otsr;
  }
  
  /** Setter for property otsr.
   * @param otsr New value of property otsr.
   *
   */
  public void setOtsr(adelfeToolkit.openTool.OpenToolSynthesisRequester otsr) {
    this.otsr = otsr;
  }
  
}
