/*
 * OpenToolReaderThread.java
 *
 * Created on 18 juin 2003, 09:58
 */

package adelfeToolkit.openTool;

import java.io.*;
import java.util.TimerTask;
import java.io.BufferedReader;

/** This class is used to read in the OT log files. It is a <CODE>TilerTask</CODE>
 * to constantly read in the file during the execution
 * @author IRIT - Toulouse (projet ADELFE)
 */
public class OpenToolLogReader extends TimerTask implements OpenToolMessageHandler, adelfeToolkit.utilities.LogWriter {
  
  /** the reader to read in */  
  private BufferedReader br;
  
  /** The owner */  
  private OpenToolConnector owner;
  
  /** the listeners for the <CODE>OpenToolMessageEvent</CODE> */  
  private java.util.Vector listeners = new java.util.Vector(0);
  
  /** the previous length of the current log file */  
  private long previousLength = -1;
  
  /** the previous path of log file */  
  private String previousPath = "";
  
  /** the number of the previous last line of the log file */  
  private String previousLastLine = "";
  
  /** the saved line to send */  
  private String lineSaved = "";
  
  /** Creates a new instance of OpenToolReaderThread
   * @param o the owner
   */
  public OpenToolLogReader(OpenToolConnector o) {
    owner = o;
  }
  
  /** the method which is called by a Timer. It reads the log file and extract the
   * last line of the file if it has changed since the last reading.
   */  
  public void run() {
    String s;
    if (owner.getOwner().getManagerModule() != null) {
      adelfeToolkit.activities.Project p = owner.getOwner().getManagerModule().getCurrentProject();
      String dir = p.getDirectory().getFile()+p.getActionName()+"_OT";
      String filesep = System.getProperty("file.separator");
      File f = new File(dir+filesep+p.getActionName()+".log");
      
      if (f.exists()) {
        if (previousPath.equals(f.getAbsolutePath())){
          long l = f.length();
          if (l > previousLength) {
            previousLength = l;
            readFile(f);
          }
          else {
          }
        } 
        else {
          previousPath = f.getAbsolutePath();
          previousLength = 0;
        }
      }
      else {
      }
    }
  }
  
  /** this method reads the last line of a OT log file if necesary (if there was any
   * change since before). If a line is found, an <CODE>OpenToolMessageEvent</CODE>
   * is send to listeners
   * @param f the log file to read
   */  
  private void readFile(File f) {
    try {
      br = new BufferedReader(new java.io.FileReader(f));
      int res = goToLastLine();
      if (res == 0) {
        previousLastLine = br.readLine();
      }
      else {
        previousLastLine = lineSaved;
      }
      while (previousLastLine != null) {
        processLine(previousLastLine);
        previousLastLine = br.readLine();
      }
      br.close();
    } catch (java.io.FileNotFoundException e1) {
      printErrLog("readFile::"+e1);
    } catch (java.io.IOException e2) {
      printErrLog("readFile::"+e2);
    }
  }
  
  /** this method extract the last line of the log file
   * @return the last line of the log file
   */  
  private int goToLastLine(){
    boolean lastLineReached = false;
    int res = 0;
    try {
      while (!lastLineReached) {
        String s = br.readLine();
        if (s == null) {
          lastLineReached = true;
          res = 1;
        } else {
          lineSaved = s;
          lastLineReached = s.equals(this.previousLastLine);
        }
      }
    } catch (java.io.IOException e2) {
      printErrLog("goToLastLine::"+e2);
    }
    return res;
  }
  
  /** this method is called when a new line is found. It sends an
   * <CODE>OpenToolMessageEvent</CODE> to listeners
   * @param line the new line
   */  
  private void processLine(String line) {
    this.fireOpenToolMessageReceived(new OpenToolMessageEvent(this,line));
  }
  
  /** Getter for property br.
   * @return Value of property br.
   *
   */
  public BufferedReader getBr() {
    return br;
  }
  
  /** Setter for property br.
   * @param br New value of property br.
   *
   */
  public void setBr(BufferedReader br) {
    this.br = br;
  }
  
  /** it adds a new listener to the listeners list
   * @param listener the listener to add
   */  
  public void addOpenToolMessageListener(OpenToolMessageListener listener) {
    this.listeners.addElement(listener);
  }
  
  /** it sends an <CODE>OpenToolMessageEvent</CODE> to all the listeners
   * @param evt the event to send
   */  
  public void fireOpenToolMessageReceived(OpenToolMessageEvent evt) {
    int i;
    for(i=0;i<this.listeners.size();i++)
      ((OpenToolMessageListener) this.listeners.get(i)).openToolMessageReceived(evt);
  }
  
  /** it removes a listener form the list
   * @param listener the listener to remove
   */  
  public void removeOpenToolMessageListener(OpenToolMessageListener listener) {
    this.listeners.removeElement(listener);
  }
  
  /** prints an error in the Adelfe log file
   * @param s the error to print
   * @see adelfeToolkit.utilities.LogWriter
   */  
  public void printErrLog(String s) {
    this.owner.getOwner().printlnLogFileErr("OpenToolLogReader::"+s);
  }
  
  /** prints a message in the Adelfe log file
   * @param s the message to print
   * @see adelfeToolkit.utilities.LogWriter
   */  
  public void printOutLog(String s) {
    this.owner.getOwner().printlnLogFileOut("OpenToolLogReader::"+s);
  }
  
}
